@@ -5,7 +5,7 @@ import logging
 import yaml
 import click
 import paho.mqtt.client as mqtt
-from modbus_interface import modbus_interface
+from . import modbus_interface
 
 class mqtt_interface():
     def __init__(self, hostname, port, username, password, config_file, mqtt_topic_prefix):
@@ -22,21 +22,21 @@ class mqtt_interface():
 
     def connect(self):
         # Connects to modbus and MQTT.
-        self.mb = modbus_interface(self.config['ip'], self.config['port'], self.config['update_rate'])
+        self.mb = modbus_interface.modbus_interface(self.config['ip'], self.config['port'], self.config['update_rate'])
         self.mb.connect()
         # Tells the modbus interface about the registers we consider interesting.
         for register in self.registers:
-            self.mb.add_monitor_register(register['table'], register['address'])
+            self.mb.add_monitor_register(register.get('table', 'holding'), register['address'])
             register['value'] = None
 
-        self.client = mqtt.Client()
-        self.client.username_pw_set(self.username, self.password)
-        self.client.on_connect = self.on_connect
-        self.client.on_disconnect = self.on_disconnect
-        self.client.on_message = self.on_message
-        self.client.on_subscribe = self.on_subscribe
-        self.client.connect(self.hostname, self.port, 60)
-        self.client.loop_start()
+        self._mqtt_client = mqtt.Client()
+        self._mqtt_client.username_pw_set(self.username, self.password)
+        self._mqtt_client.on_connect = self.on_connect
+        self._mqtt_client.on_disconnect = self.on_disconnect
+        self._mqtt_client.on_message = self.on_message
+        self._mqtt_client.on_subscribe = self.on_subscribe
+        self._mqtt_client.connect(self.hostname, self.port, 60)
+        self._mqtt_client.loop_start()
 
     def get_registers_with(self, required_key):
         # Returns the registers containing the required_key
@@ -44,9 +44,13 @@ class mqtt_interface():
 
     def poll(self):
         self.mb.poll()
-        self.client.publish(self.prefix+'modbus4mqtt', 'poll')
+        self._mqtt_client.publish(self.prefix+'modbus4mqtt', 'poll')
         for register in self.get_registers_with('pub_topic'):
-            value = self.mb.get_value(register['table'], register['address'])
+            try:
+                value = self.mb.get_value(register.get('table', 'holding'), register['address'])
+            except:
+                logging.warning("Couldn't get value from register {} in table {}".format(register['address'], register.get('table', 'holding')))
+                continue
             changed = False
             if value != register['value']:
                 changed = True
@@ -59,18 +63,19 @@ class mqtt_interface():
                     # This is a bit weird...
                     value = [human for human, raw in register['value_map'].items() if raw == value][0]
             retain = register.get('retain', False)
-            self.client.publish(self.prefix+register['pub_topic'], value, retain=retain)
+            self._mqtt_client.publish(self.prefix+register['pub_topic'], value, retain=retain)
 
     def on_connect(self, client, userdata, flags, rc):
         if rc == 0:
             logging.info("Connected to MQTT.")
         else:
             logging.error("Couldn't connect to MQTT.")
+            return
         # Subscribe to all the set topics.
         for register in self.get_registers_with('set_topic'):
-            self.client.subscribe(self.prefix+register['set_topic'])
+            self._mqtt_client.subscribe(self.prefix+register['set_topic'])
             print("Subscribed to {}".format(self.prefix+register['set_topic']))
-        self.client.publish(self.prefix+'modbus4mqtt', 'hi')
+        self._mqtt_client.publish(self.prefix+'modbus4mqtt', 'hi')
 
     def on_disconnect(self, client, userdata, flags, rc):
         print("Disconnected")
@@ -79,8 +84,7 @@ class mqtt_interface():
         pass
 
     def on_message(self, client, userdata, msg):
-        # print("got a message:")
-        print("got a message: {}: {}".format(msg.topic, msg.payload))
+        # print("got a message: {}: {}".format(msg.topic, msg.payload))
         topic = msg.topic[len(self.prefix):]
         for register in [register for register in self.registers if 'set_topic' in register]:
             if topic != register['set_topic']:
@@ -93,7 +97,12 @@ class mqtt_interface():
                     continue
                 # Map the value from the human-readable form into the raw modbus number
                 value = register['value_map'][value]
-            self.mb.set_value(register['table'], register['address'], int(value))
+            try:
+                value = int(value)
+            except ValueError:
+                logging.error("Failed to convert register value for writing. Bad/missing value_map? Topic: {}, Value: {}".format(topic, value))
+                continue
+            self.mb.set_value(register.get('table', 'holding'), register['address'], int(value))
 
     def load_modbus_config(self, path):
         return yaml.load(open(path,'r').read(), Loader=yaml.FullLoader)
